#include "rendering/Renderer.h"
#include "gaming-core/wrapper/GL.h"

Renderer::Renderer(Shader& shader) : shader(shader), texture(true), color(true) {
    shader.use();
    GL::enableBlending();
    setTextureMode(false);
    setColorMode(false);
}

void Renderer::pop() {
    stack.pop();
}

void Renderer::push() {
    stack.push();
}

void Renderer::update() {
    shader.setMatrix("model", stack.peek().getValues());
}

Renderer& Renderer::scale(float sx, float sy) {
    stack.peek().scale(Vector3(sx, sy, 1.0f));
    return *this;
}

Renderer& Renderer::scale(float s) {
    stack.peek().scale(s);
    return *this;
}

Renderer& Renderer::translate(float tx, float ty) {
    stack.peek().translateX(tx).translateY(ty);
    return *this;
}

Renderer& Renderer::translateX(float tx) {
    stack.peek().translateX(tx);
    return *this;
}

Renderer& Renderer::translateY(float ty) {
    stack.peek().translateY(ty);
    return *this;
}

Renderer& Renderer::translateTo(float tx, float ty) {
    stack.peek().translateTo(Vector3(tx, ty, 0.0f));
    return *this;
}

Renderer& Renderer::rotate(float degrees) {
    stack.peek().rotateZ(degrees);
    return *this;
}

void Renderer::setTextureMode(bool b) {
    if(b != texture) {
        texture = b;
        shader.setInt("useTexture", b);
    }
}

void Renderer::setColorMode(bool b) {
    if(b != color) {
        color = b;
        shader.setInt("useColor", b);
    }
}

void Renderer::setStringSize(int size) {
    fontRenderer.setSize(size);
}

float Renderer::drawString(float x, float y, const char* text) {
    setTextureMode(true);
    setColorMode(true);
    return fontRenderer.drawString(x, y, text);
}

void Renderer::drawTriangle(const ColorRenderer::Vertex& v1, const ColorRenderer::Vertex& v2, const ColorRenderer::Vertex& v3) {
    setTextureMode(false);
    setColorMode(true);
    colorRenderer.draw(v1, v2, v3);
}

void Renderer::drawTriangle(float x1, float y1, float x2, float y2, float x3, float y3, Color4 color) {
    ColorRenderer::Vertex v1 = {x1, y1, color};
    ColorRenderer::Vertex v2 = {x2, y2, color};
    ColorRenderer::Vertex v3 = {x3, y3, color};
    drawTriangle(v1, v2, v3);
}

void Renderer::drawRectangle(float x, float y, float width, float height, Color4 color) {
    drawTriangle(x, y, x + width, y, x, y + height, color);
    drawTriangle(x + width, y, x, y + height, x + width, y + height, color);
}